# Copyright (C) 2008, OLPC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gtk
import gobject
from gettext import gettext as _

from sugar.graphics import style
from sugar.graphics.alert import Alert
from sugar.graphics.icon import Icon

from jarabe.controlpanel.sectionview import SectionView
from jarabe.controlpanel.inlinealert import InlineAlert


CLASS = 'Network'
ICON = 'module-network'
TITLE = _('Network')

_APPLY_TIMEOUT = 3000


# "Proxy" feature merged into "Network"
#################################################################################################################

# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _
import re

import gconf

_widget_sensitivies = {}
_gconf_origin_values = {}


class Proxy(SectionView):

    def __init__(self):
        SectionView.__init__(self)
        self.set_border_width(style.DEFAULT_SPACING * 2)
        self.set_spacing(style.DEFAULT_SPACING)
        self.setup()

    def setup(self):
        for i in self.get_children():
            self.remove(i)
            # Destroy all widgets and connection to avoid any interfering
            i.destroy()

        _widget_sensitivies.clear()

        workspace = gtk.VBox()
        workspace.show()
        self.add(workspace)

        def add_section(section, label_text):
            separator = gtk.HSeparator()
            separator.show()
            workspace.pack_start(separator, expand=False)

            label = gtk.Label(label_text)
            label.set_alignment(0, 0)
            label.show()
            workspace.pack_start(label, expand=False)

            section.set_border_width(style.DEFAULT_SPACING * 2)
            section.show()
            workspace.pack_start(section, expand=False)

        add_section(_ProxySection(),
                _('Configure proxies to access the internet'))
        add_section(_IgnoreSection(), _('Ignore host list'))

    def undo(self):
        conf = gconf.client_get_default()
        for key, value in _gconf_origin_values.items():
            if value is None:
                conf.unset(key)
            else:
                conf.set(key, value)

    @property
    def needs_restart(self):
        conf = gconf.client_get_default()
        for key, value in _gconf_origin_values.items():
            if ((value is None and conf.get_without_default(key) is not None) or
                (value is not None and value.to_string() != conf.get(key).to_string())):
                return True

        return False

    @needs_restart.setter
    def needs_restart(self, value):
        # needs_restart is a property (i.e. gets calculated) in this Control
        # Panel, but SectionView.__init__() wants to initialise it to False,
        # so we need to provide a (fake) setter.
        pass


class _ProxySection(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)
        self._common_hosts = {}
        self._common_ports = {}

        group = gtk.RadioButton()
        group.props.label = _('Direct internet connection')
        group.show()
        self.pack_start(group, expand=False)
        _register_selector_key('/system/proxy/mode', group, 'none')
        _register_bool_key('/system/http_proxy/use_http_proxy', group, True)

        manual_proxy = gtk.RadioButton(group)
        manual_proxy.props.label = _('Manual proxy configuration')
        manual_proxy.show()
        self.pack_start(manual_proxy, expand=False)
        _register_selector_key('/system/proxy/mode', manual_proxy, 'manual')

        widgets = self._add_protos()
        manual_proxy.connect('toggled', _set_sensitive, False, widgets)
        _set_sensitive(manual_proxy, False, widgets)

        auto_proxy = gtk.RadioButton(group)
        auto_proxy.props.label = _('Automatic proxy configuration')
        auto_proxy.show()
        self.pack_start(auto_proxy, expand=False)
        _register_selector_key('/system/proxy/mode', auto_proxy, 'auto')

        grid = self._sub_section_new()
        grid.attach_label(_('Autoconfiguration URL (leave empty to use WPAD):'),
                          0, 1, 0, 1)
        entry = grid.attach_entry(1, 2, 0, 1)
        _register_string_key('/system/proxy/autoconfig_url', entry)
        auto_proxy.connect('toggled', _set_sensitive, False, [grid])
        _set_sensitive(auto_proxy, False, [grid])

        print 'aa toh raha hai'

    def _add_protos(self):
        commons = gtk.CheckButton()
        commons.props.label = _('Use the same proxy for all protocols')
        commons.show()
        self.pack_start(commons)
        _register_bool_key('/system/http_proxy/use_same_proxy', commons)

        grid = self._sub_section_new()

        def add_proto(row, is_common, label_text, host_key, port_key):
            host_label = grid.attach_label(label_text, 0, 1, row, row + 1)
            host = grid.attach_entry(1, 2, row, row + 1)

            port_label = grid.attach_label(_('Port:'), 2, 3, row, row + 1)
            port_value = gtk.Adjustment(8080, 0, 65536, 1, 10)
            port = gtk.SpinButton()
            port.configure(port_value, .1, 0)
            port.show()
            grid.attach(port, 3, 4, row, row + 1,
                    gtk.SHRINK | gtk.FILL, gtk.SHRINK)

            if is_common:
                _widget_sensitivies.update([
                        (host_label, None), (host, None),
                        (port_label, None), (port, None)])
                self._common_hosts[host] = host.props.buffer
                self._common_ports[port] = port.props.adjustment

            _register_string_key(host_key, host)
            _register_int_key(port_key, port)

            return host, port

        http_host, http_port = add_proto(1, False, _('HTTP proxy:'),
                '/system/http_proxy/host', '/system/http_proxy/port')

        auth_widget = _AuthWidget()
        auth_widget.show()
        grid.attach(auth_widget, 1, 2, 2, 3, gtk.SHRINK | gtk.FILL, gtk.SHRINK)

        add_proto(3, True, _('Secure HTTP proxy:'),
                '/system/proxy/secure_host', '/system/proxy/secure_port')
        add_proto(4, True, _('FTP proxy:'),
                '/system/proxy/ftp_host', '/system/proxy/ftp_port')
        add_proto(5, True, _('Socks proxy:'),
                '/system/proxy/socks_host', '/system/proxy/socks_port')

        def commons_toggled_cb(sender):
            for widget in _widget_sensitivies.keys():
                _widget_sensitivies[widget] = not sender.props.active
            _set_sensitive(sender, True, _widget_sensitivies.keys())

            for widget, orig_buffer in self._common_hosts.items():
                widget.props.buffer = http_host.props.buffer if \
                        sender.props.active else orig_buffer

            for widget, orig_adjustment in self._common_ports.items():
                widget.props.adjustment = http_port.props.adjustment if \
                        sender.props.active else orig_adjustment
                widget.props.value = widget.props.adjustment.value

        commons.connect('toggled', commons_toggled_cb)
        commons_toggled_cb(commons)

        return [commons, grid]

    def _sub_section_new(self):
        grid = _Grid(1, 1, False)
        grid.props.column_spacing = style.DEFAULT_SPACING
        grid.props.row_spacing = style.DEFAULT_SPACING
        grid.show()

        alignment = gtk.Alignment(0, 0, 1, 1)
        alignment.props.left_padding = style.STANDARD_ICON_SIZE
        alignment.props.right_padding = style.GRID_CELL_SIZE
        alignment.add(grid)
        alignment.show()
        self.pack_start(alignment)

        return grid


class _IgnoreSection(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)

        entry = gtk.Entry()
        entry.show()
        self.pack_start(entry, expand=False)
        _register_list_key('/system/http_proxy/ignore_hosts', entry)


class _AuthWidget(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)

        enable = gtk.CheckButton()
        enable.props.label = _('Use authentication')
        enable.show()
        self.pack_start(enable, expand=False)
        _register_bool_key('/system/http_proxy/use_authentication', enable)

        grid = _Grid(2, 2, False)
        grid.props.column_spacing = style.DEFAULT_SPACING
        grid.props.row_spacing = style.DEFAULT_SPACING
        self.pack_start(grid)

        grid.attach_label(_('Username:'), 0, 1, 0, 1)
        entry = grid.attach_entry(1, 2, 0, 1)
        _register_string_key('/system/http_proxy/authentication_user', entry)

        grid.attach_label(_('Password:'), 0, 1, 1, 2)
        entry = grid.attach_entry(1, 2, 1, 2)
        entry.props.visibility = False
        _register_string_key(
                '/system/http_proxy/authentication_password', entry)

        enable.connect('toggled', lambda sender:
                grid.show() if sender.props.active else grid.hide())
        if enable.props.active:
            grid.show()


class _Grid(gtk.Table):

    def attach_label(self, label, left_attach, right_attach,
            top_attach, bottom_attach):
        widget = gtk.Label(label)
        widget.set_alignment(0, 0)
        widget.set_line_wrap(True)
        self.attach(widget, left_attach, right_attach,
                top_attach, bottom_attach, gtk.SHRINK | gtk.FILL, gtk.SHRINK)
        widget.show()
        return widget

    def attach_entry(self, left_attach, right_attach,
            top_attach, bottom_attach):
        widget = gtk.Entry()
        self.attach(widget, left_attach, right_attach,
                top_attach, bottom_attach, gtk.EXPAND | gtk.FILL, gtk.SHRINK)
        widget.show()
        return widget


def _set_sensitive(sender, reverse, widgets):
    is_sensitive = sender.props.active
    if reverse:
        is_sensitive = not is_sensitive

    for i in widgets:
        if isinstance(i, gtk.Container):
            _set_sensitive(sender, reverse, i.get_children())
        i.props.sensitive = is_sensitive and _widget_sensitivies.get(i, True)


def _register_bool_key(key, widget, reverse=False):

    def set_cb(widget, x, reverse):
        value = x.get_bool()
        if reverse:
            value = not value
        widget.props.active = value

    def get_cb(widget, reverse):
        x = gconf.Value(gconf.VALUE_BOOL)
        value = widget.props.active
        if reverse:
            value = not value
        x.set_bool(value)
        return x

    _register_key(key, widget, 'toggled', set_cb, get_cb, reverse)


def _register_string_key(key, widget):

    def set_cb(widget, x):
        widget.props.text = x.get_string()

    def get_cb(widget):
        x = gconf.Value(gconf.VALUE_STRING)
        x.set_string(widget.props.text)
        return x

    _register_key(key, widget, 'changed', set_cb, get_cb)


def _register_int_key(key, widget):

    def set_cb(widget, x):
        widget.props.value = x.get_int()

    def get_cb(widget):
        x = gconf.Value(gconf.VALUE_INT)
        x.set_int(int(widget.props.value))
        return x

    _register_key(key, widget.props.adjustment, 'value_changed',
            set_cb, get_cb)


def _register_selector_key(key, widget, value):

    def set_cb(widget, x, value):
        widget.props.active = x.get_string() == value

    def get_cb(widget, value):
        if not widget.props.active:
            return None
        x = gconf.Value(gconf.VALUE_STRING)
        x.set_string(value)
        return x

    _register_key(key, widget, 'toggled', set_cb, get_cb, value)


def _register_list_key(key, widget):

    def set_cb(widget, x):
        hosts = [i.get_string() for i in x.get_list()]
        widget.props.text = ', '.join(hosts)

    def get_cb(widget):
        hosts = []
        for i in re.split('[\s,;:]+', widget.props.text or ''):
            if not i.strip():
                continue
            value = gconf.Value(gconf.VALUE_STRING)
            value.set_string(i.strip())
            hosts.append(value)
        x = gconf.Value(gconf.VALUE_LIST)
        x.set_list_type(gconf.VALUE_STRING)
        x.set_list(hosts)
        return x

    _register_key(key, widget, 'changed', set_cb, get_cb)


def _register_key(key, widget, signal, set_cb, get_cb, *args):
    conf = gconf.client_get_default()
    value = conf.get(key)
    if value is not None:
        set_cb(widget, value, *args)

    _gconf_origin_values[key] = value

    def signal_cb(sender, key, widget, get_cb, *args):
        value = get_cb(widget, *args)
        if value is not None:
            conf = gconf.client_get_default()
            conf.set(key, value)

    widget.connect(signal, signal_cb, key, widget, get_cb, *args)
#################################################################################################################
# "Proxy" code-merging finished !!




class AddRemoveWidget(gtk.HBox):

    def __init__(self, label, add_button_clicked_cb,
                 remove_button_clicked_cb, index):
        gtk.Box.__init__(self)
        self.set_homogeneous(False)
        self.set_spacing(10)

        self._index = index
        self._add_button_added = False
        self._remove_button_added = False

        self._entry_box = gtk.Entry()
        self._entry_box.set_text(label)
        self.pack_start(self._entry_box, expand=False)
        self._entry_box.show()

        add_icon = Icon(icon_name='list-add')
        self._add_button = gtk.Button()
        self._add_button.set_image(add_icon)
        self._add_button.connect('clicked',
                                 add_button_clicked_cb,
                                 self)

        remove_icon = Icon(icon_name='list-remove')
        self._remove_button = gtk.Button()
        self._remove_button.set_image(remove_icon)
        self._remove_button.connect('clicked',
                                    remove_button_clicked_cb,
                                    self)

        self.__add_add_button()
        self.__add_remove_button()

    def _get_index(self):
        return self._index

    def _set_index(self, value):
        self._index = value

    def _get_entry(self):
        return self._entry_box.get_text()

    def __add_add_button(self):
        self.pack_start(self._add_button, expand=False)
        self._add_button.show()
        self._add_button_added = True

    def _remove_remove_button_if_not_already(self):
        if self._remove_button_added:
            self.__remove_remove_button()

    def __remove_remove_button(self):
        self.remove(self._remove_button)
        self._remove_button_added = False

    def _add_remove_button_if_not_already(self):
        if not self._remove_button_added:
            self.__add_remove_button()

    def __add_remove_button(self):
        self.pack_start(self._remove_button, expand=False)
        self._remove_button.show()
        self._remove_button_added = True


class MultiWidget(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)

    def _add_widget(self, label):
        new_widget = AddRemoveWidget(label,
                                     self.__add_button_clicked_cb,
                                     self.__remove_button_clicked_cb,
                                     len(self.get_children()))
        self.add(new_widget)
        new_widget.show()
        self.show()
        self._update_remove_button_statuses()

    def __add_button_clicked_cb(self, add_button,
                                      add_button_container):
        self._add_widget('')
        self._update_remove_button_statuses()

    def __remove_button_clicked_cb(self, remove_button,
                                   remove_button_container):
        for child in self.get_children():
            if child._get_index() > remove_button_container._get_index():
                child._set_index(child._get_index() - 1)

        self.remove(remove_button_container)
        self._update_remove_button_statuses()

    def _update_remove_button_statuses(self):
        children = self.get_children()

        # Now, if there is only one entry, remove-button
        # should not be shown.
        if len(children) == 1:
            children[0]._remove_remove_button_if_not_already()

        # Alternatively, if there are more than 1 entries,
        # remove-button should be shown for all.
        if len(children) > 1:
            for child in children:
                child._add_remove_button_if_not_already()


    def _get_entries(self):
        entries = []
        for child in self.get_children():
            entries.append(child._get_entry())

        return entries


class Network(SectionView):
    def __init__(self, model, alerts):
        SectionView.__init__(self)

        self._model = model
        self.restart_alerts = alerts
        self._jabber_sid = 0
        self._jabber_valid = True
        self._radio_valid = True
        self._jabber_change_handler = None
        self._radio_change_handler = None
        self._network_configuration_reset_handler = None

        self.set_border_width(style.DEFAULT_SPACING * 2)
        self.set_spacing(style.DEFAULT_SPACING)
        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        self._radio_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._jabber_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._hidden_network_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)

        workspace = gtk.VBox()
        workspace.show()

        separator_wireless = gtk.HSeparator()
        workspace.pack_start(separator_wireless, expand=False)
        separator_wireless.show()

        label_wireless = gtk.Label(_('Wireless'))
        label_wireless.set_alignment(0, 0)
        workspace.pack_start(label_wireless, expand=False)
        label_wireless.show()
        box_wireless = gtk.VBox()
        box_wireless.set_border_width(style.DEFAULT_SPACING * 2)
        box_wireless.set_spacing(style.DEFAULT_SPACING)

        radio_info = gtk.Label(_('Turn off the wireless radio to save battery'
                                 ' life'))
        radio_info.set_alignment(0, 0)
        radio_info.set_line_wrap(True)
        radio_info.show()
        box_wireless.pack_start(radio_info, expand=False)

        box_radio = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._button = gtk.CheckButton()
        self._button.set_alignment(0, 0)
        box_radio.pack_start(self._button, expand=False)
        self._button.show()

        label_radio = gtk.Label(_('Radio'))
        label_radio.set_alignment(0, 0.5)
        box_radio.pack_start(label_radio, expand=False)
        label_radio.show()

        box_wireless.pack_start(box_radio, expand=False)
        box_radio.show()

        self._radio_alert = InlineAlert()
        self._radio_alert_box.pack_start(self._radio_alert, expand=False)
        box_radio.pack_end(self._radio_alert_box, expand=False)
        self._radio_alert_box.show()
        if 'radio' in self.restart_alerts:
            self._radio_alert.props.msg = self.restart_msg
            self._radio_alert.show()

        history_info = gtk.Label(_('Discard network history if you have'
                                   ' trouble connecting to the network'))
        history_info.set_alignment(0, 0)
        history_info.set_line_wrap(True)
        history_info.show()
        box_wireless.pack_start(history_info, expand=False)

        box_clear_history = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._clear_history_button = gtk.Button()
        self._clear_history_button.set_label(_('Discard network history'))
        box_clear_history.pack_start(self._clear_history_button, expand=False)
        if not self._model.have_networks():
            self._clear_history_button.set_sensitive(False)
        self._clear_history_button.show()
        box_wireless.pack_start(box_clear_history, expand=False)
        box_clear_history.show()

        workspace.pack_start(box_wireless, expand=False)
        box_wireless.show()

        separator_mesh = gtk.HSeparator()
        workspace.pack_start(separator_mesh, False)
        separator_mesh.show()

        label_mesh = gtk.Label(_('Collaboration'))
        label_mesh.set_alignment(0, 0)
        workspace.pack_start(label_mesh, expand=False)
        label_mesh.show()
        box_mesh = gtk.VBox()
        box_mesh.set_border_width(style.DEFAULT_SPACING * 2)
        box_mesh.set_spacing(style.DEFAULT_SPACING)

        server_info = gtk.Label(_("The server is the equivalent of what"
                                  " room you are in; people on the same server"
                                  " will be able to see each other, even when"
                                  " they aren't on the same network."))
        server_info.set_alignment(0, 0)
        server_info.set_line_wrap(True)
        box_mesh.pack_start(server_info, expand=False)
        server_info.show()

        box_server = gtk.HBox(spacing=style.DEFAULT_SPACING)
        label_server = gtk.Label(_('Server:'))
        label_server.set_alignment(1, 0.5)
        label_server.modify_fg(gtk.STATE_NORMAL,
                               style.COLOR_SELECTION_GREY.get_gdk_color())
        box_server.pack_start(label_server, expand=False)
        group.add_widget(label_server)
        label_server.show()
        self._entry = gtk.Entry()
        self._entry.set_alignment(0)
        self._entry.modify_bg(gtk.STATE_INSENSITIVE,
                        style.COLOR_WHITE.get_gdk_color())
        self._entry.modify_base(gtk.STATE_INSENSITIVE,
                          style.COLOR_WHITE.get_gdk_color())
        self._entry.set_size_request(int(gtk.gdk.screen_width() / 3), -1)
        box_server.pack_start(self._entry, expand=False)
        self._entry.show()
        box_mesh.pack_start(box_server, expand=False)
        box_server.show()

        self._jabber_alert = InlineAlert()
        label_jabber_error = gtk.Label()
        group.add_widget(label_jabber_error)
        self._jabber_alert_box.pack_start(label_jabber_error, expand=False)
        label_jabber_error.show()
        self._jabber_alert_box.pack_start(self._jabber_alert, expand=False)
        box_mesh.pack_end(self._jabber_alert_box, expand=False)
        self._jabber_alert_box.show()
        if 'jabber' in self.restart_alerts:
            self._jabber_alert.props.msg = self.restart_msg
            self._jabber_alert.show()

        workspace.pack_start(box_mesh, expand=False)
        box_mesh.show()

        if self._model.is_hidden_network_connect_package_available():
            separator_hidden_network = gtk.HSeparator()
            workspace.pack_start(separator_hidden_network, False)
            separator_hidden_network.show()

            label_hidden_network = gtk.Label(_('Hidden Networks'))
            label_hidden_network.set_alignment(0, 0)
            workspace.pack_start(label_hidden_network, expand=False)
            label_hidden_network.show()
            box_hidden_network = gtk.VBox()
            box_hidden_network.set_border_width(style.DEFAULT_SPACING * 2)
            box_hidden_network.set_spacing(style.DEFAULT_SPACING)

            info = gtk.Label(_("Enter the SSIDs of the hidden networks."))
            info.set_alignment(0, 0)
            info.set_line_wrap(True)
            box_hidden_network.pack_start(info, expand=False)
            info.show()

            self._hidden_network_alert = InlineAlert()
            self.hidden_network_msg = 'XO-reboot is required for the' \
                                      ' changes to take effect.'
            self._hidden_network_alert.props.msg =_(self.hidden_network_msg)
            self._hidden_network_alert_box.pack_start(self._hidden_network_alert,
                                                        expand=False)
            box_hidden_network.pack_end(self._hidden_network_alert_box,
                                                expand=False)
            self._hidden_network_alert_box.show()
            self._hidden_network_alert.show()

            self._widget_table = MultiWidget()
            box_hidden_network.pack_start(self._widget_table, expand=False)
            self._widget_table.show()

            save_button = gtk.Button()
            save_button.set_alignment(0, 0)
            save_button.set_label('Save')
            save_button.connect('clicked', self.__save_button_clicked_cb)
            box_save_button = gtk.HBox()
            box_save_button.set_homogeneous(False)
            box_save_button.pack_start(save_button, expand=False)
            save_button.show()

            box_hidden_network.pack_start(box_save_button, expand=False)
            box_save_button.show()

            workspace.pack_start(box_hidden_network, expand=False)
            box_hidden_network.show()


        separator_proxy = gtk.HSeparator()
        workspace.pack_start(separator_proxy, False)
        separator_proxy.show()

        label_proxy = gtk.Label(_('Proxy'))
        label_proxy.set_alignment(0, 0)
        workspace.pack_start(label_proxy, expand=False)
        label_proxy.show()

        box_proxy = gtk.VBox()
        box_proxy.set_border_width(style.DEFAULT_SPACING * 2)
        box_proxy.set_spacing(style.DEFAULT_SPACING)
        proxy_info = gtk.Label(_("<Enter suitable info>"))
        proxy_info.set_alignment(0, 0)
        proxy_info.set_line_wrap(True)
        box_proxy.pack_start(proxy_info, expand=False)
        proxy_info.show()
        workspace.pack_start(box_proxy, expand=False)
        box_proxy.show()

        proxy = Proxy()
        workspace.pack_start(proxy, expand=False)
        proxy.show()

        separator_nm_connection_editor = gtk.HSeparator()
        workspace.pack_start(separator_nm_connection_editor, False)
        separator_nm_connection_editor.show()

        label_nm_connection_editor = gtk.Label(_('NetworkManager'
                                                 ' Connection Editor'))
        label_nm_connection_editor.set_alignment(0, 0)
        workspace.pack_start(label_nm_connection_editor, expand=False)
        label_nm_connection_editor.show()

        box_nm_connection_editor = gtk.VBox()
        box_nm_connection_editor.set_border_width(style.DEFAULT_SPACING * 2)
        box_nm_connection_editor.set_spacing(style.DEFAULT_SPACING)

        info = gtk.Label(_("Press the 'Launch' button, to launch the"
                           " Connection Editor."))
        info.set_alignment(0, 0)
        info.set_line_wrap(True)
        box_nm_connection_editor.pack_start(info, expand=False)

        launch_button = gtk.Button()
        launch_button.set_alignment(0, 0)
        launch_button.set_label(_('Launch'))
        launch_button.connect('clicked', self.__launch_button_clicked_cb)
        box_launch_button = gtk.HBox()
        box_launch_button.set_homogeneous(False)
        box_launch_button.pack_start(launch_button, expand=False)
        box_launch_button.show_all()

        box_nm_connection_editor.pack_start(box_launch_button, expand=False)
        workspace.pack_start(box_nm_connection_editor, expand=False)
        box_nm_connection_editor.show_all()


        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled.add_with_viewport(workspace)
        scrolled.show()
        self.add(scrolled)

        self.setup()

    def setup(self):
        self._entry.set_text(self._model.get_jabber())
        try:
            radio_state = self._model.get_radio()
        except self._model.ReadError, detail:
            self._radio_alert.props.msg = detail
            self._radio_alert.show()
        else:
            self._button.set_active(radio_state)

        self._jabber_valid = True
        self._radio_valid = True
        self.needs_restart = False
        self._radio_change_handler = self._button.connect( \
                'toggled', self.__radio_toggled_cb)
        self._jabber_change_handler = self._entry.connect( \
                'changed', self.__jabber_changed_cb)
        self._network_configuration_reset_handler =  \
                self._clear_history_button.connect( \
                        'clicked', self.__network_configuration_reset_cb)

        if self._model.is_hidden_network_connect_package_available():

            ssids = self._model.get_ssids()
            if len(ssids) == 0:
                self._widget_table._add_widget('')
            else:
                for ssid in ssids:
                    self._widget_table._add_widget(ssid)

    def undo(self):
        self._button.disconnect(self._radio_change_handler)
        self._entry.disconnect(self._jabber_change_handler)
        self._model.undo()
        self._jabber_alert.hide()
        self._radio_alert.hide()

    def _validate(self):
        if self._jabber_valid and self._radio_valid:
            self.props.is_valid = True
        else:
            self.props.is_valid = False

    def __radio_toggled_cb(self, widget, data=None):
        radio_state = widget.get_active()
        try:
            self._model.set_radio(radio_state)
        except self._model.ReadError, detail:
            self._radio_alert.props.msg = detail
            self._radio_valid = False
        else:
            self._radio_valid = True
            if self._model.have_networks():
                self._clear_history_button.set_sensitive(True)

        self._validate()
        return False

    def __jabber_changed_cb(self, widget, data=None):
        if self._jabber_sid:
            gobject.source_remove(self._jabber_sid)
        self._jabber_sid = gobject.timeout_add(_APPLY_TIMEOUT,
                                               self.__jabber_timeout_cb,
                                               widget)

    def __jabber_timeout_cb(self, widget):
        self._jabber_sid = 0
        if widget.get_text() == self._model.get_jabber:
            return
        try:
            self._model.set_jabber(widget.get_text())
        except self._model.ReadError, detail:
            self._jabber_alert.props.msg = detail
            self._jabber_valid = False
            self._jabber_alert.show()
            self.restart_alerts.append('jabber')
        else:
            self._jabber_valid = True
            self._jabber_alert.hide()

        self._validate()
        return False

    def __network_configuration_reset_cb(self, widget):
        # FIXME: takes effect immediately, not after CP is closed with
        # confirmation button
        self._model.clear_networks()
        if not self._model.have_networks():
            self._clear_history_button.set_sensitive(False)

    def __save_button_clicked_cb(self, save_button):
        self._model.set_ssids(self._widget_table._get_entries())
        save_button.set_sensitive(False)

    def __launch_button_clicked_cb(self, launch_button):
        self._model.launch_nm_connection_editor()
